#ifndef TORUS_H
#define TORUS_H

#include <math.h>

#include <vector>

#include "m3dmath.h"

namespace TORUS_SPACE
{
 class vector3 {
  public:
   union {
    struct {
     float x,y,z;
    };
    float v[3];
   };

   // konstruktory
   vector3(float x=0.0f,float y=0.0f,float z=0.0f):x(x),y(y),z(z) {}
   vector3(const vector3 &vec):x(vec.x),y(vec.y),z(vec.z) {}
   vector3(const vector3 &a,const vector3 &b):x(b.x-a.x),y(b.y-a.y),z(b.z-a.z) {}
   vector3(const float *v):x(v[0]),y(v[1]),z(v[2]) {}

   // vector set
   void set(float nx,float ny,float nz) { x=nx; y=ny; z=nz; }
   // vector index
   float &operator[](const long idx) { return *((&x)+idx); }
   // vector assignment
   const vector3 &operator=(const vector3 &vec) { x=vec.x; y=vec.y; z=vec.z; return *this; }
   // vector equality
   const bool operator==(const vector3 &vec) const { return ((x==vec.x)&&(y==vec.y)&&(z==vec.z)); }
   // vector inequality
   const bool operator!=(const vector3 &vec) const { return !(*this==vec); }
   // vector add
   const vector3 operator+(const vector3 &vec) const { return vector3(x+vec.x,y+vec.y,z+vec.z); }
   // vector add (opposite of negation)
   const vector3 operator+() const { return vector3(*this); }
   // vector increment
   const vector3& operator+=(const vector3& vec) { x+=vec.x; y+=vec.y; z+=vec.z; return *this; }
   // vector subtraction
   const vector3 operator-(const vector3& vec) const { return vector3(x-vec.x,y-vec.y,z-vec.z); }
   // vector negation
   const vector3 operator-() const { return vector3(-x,-y,-z); }
   // vector decrement
   const vector3 &operator-=(const vector3& vec) { x-=vec.x; y-=vec.y; z-=vec.z; return *this; }
   // scalar self-multiply
   const vector3 &operator*=(const float s) { x*=s; y*=s; z*=s; return *this; }
   // scalar self-divide
   const vector3 &operator/=(const float s) {
    const float recip=1.0f/s;
    x*=recip; y*=recip; z*=recip;
    return *this;
   }
   // post multiply by scalar
   const vector3 operator*(const float s) const { return vector3(x*s,y*s,z*s); }
   // pre multiply by scalar
   friend const vector3 operator*(const float s, const vector3& vec) { return vec*s; }
   // divide by scalar
   const vector3 operator/(const float s) const {
    const float recip=1.0f/s;
    return vector3(recip*x,recip*y,recip*z);
   }
   // vector multiply
   const vector3 operator*(const vector3& vec) const { return vector3(x*vec.x,y*vec.y,z*vec.z); }
   // cross product
   const vector3 CrossProduct(const vector3 &vec) const {
    return vector3(y*vec.z-z*vec.y,z*vec.x-x*vec.z,x*vec.y-y*vec.x);
   }
   // dot product
   const float DotProduct(const vector3 &vec) const {
    return x*vec.x+y*vec.y+z*vec.z;
   }
   // length of vector
   const float Length() const { return sqrtf(x*x+y*y+z*z); }
   // return the unit vector
   const vector3 UnitVector() const { return (*this)/Length(); }
   // normalize this vector
   void Normalize() { (*this)/=Length(); }
   // return angle between two vectors
   const float inline Angle(const vector3& vec) const { return acosf(DotProduct(vec)); }
   // reflect this vector off surface with normal vector
   const vector3 Reflection(const vector3& normal) const {
    return (*this-normal*2.0f*DotProduct(normal));
   }
   // rotate angle degrees about a normal
   const vector3 Rotate(const float angle, const vector3& normal) const {
    const float cosine=cosf(angle);
    const float sine=sinf(angle);
    return vector3(*this*cosine+((normal* *this)*(1.0f-cosine))*normal+(CrossProduct(normal))*sine);
   }
 };
}

struct TorusVertex {
 TORUS_SPACE::vector3 position;
 float s,t;
 TORUS_SPACE::vector3 tangent,binormal,normal;
 // pole tangentov, binormal a normal
 TORUS_SPACE::vector3 tbn[3];
};

class Torus {
 public:
  Torus(int precision); // konstruktor
  ~Torus();             // destruktor

  std::vector <TorusVertex> vertices;
  std::vector <unsigned int> indices;
};

#endif
